<stylesheet xmlns="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	version="2.0">
<import href="generator/seaf-commons/CommonBusinessFunctions.xsl" />

<!-- Full types -->

<function name="per:entity-criteria-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal($entity,'.domain','Criteria')" />
</function>

<function name="per:entity-bean-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal($entity,'.domain.bean','Bean')" />
</function>

<function name="per:entity-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal($entity,'.domain','')" />
</function>

<function name="per:hbm-xml-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<choose>
	<when test="$entity/@generate-entity='entity-only'">
		<value-of select="per:entity-full-type($entity)" />
	</when>
	<otherwise>
		<value-of select="per:entity-bean-full-type($entity)" />
	</otherwise>
	</choose>
</function>

<function name="per:dao-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal($entity,'.dao','DAO')" />
</function>

<function name="per:hibernate-dao-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of 
		select="per:entity-type-internal($entity,'.dao.hibernate','HibernateDAO')" />
</function>

<function name="per:controller-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:controller-full-type($entity,$entity/@name)" />
</function>

<function name="per:controller-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<param name="entity-name" as="xs:string" />
	<value-of select="per:entity-type-internal(
			$entity,$entity-name,'.controller','ServerController')" />
</function>

<function name="per:controller-base-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal(
		$entity,'.controller','ServerControllerBase')" />
</function>

<function name="per:controller-impl-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:controller-impl-full-type($entity,$entity/@name)" />
</function>

<function name="per:controller-impl-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<param name="entity-name" as="xs:string" />
	<value-of select="per:entity-type-internal(
			$entity,$entity-name,'.controller.impl','ServerControllerImpl')" />
</function>

<function name="per:controller-impl-base-full-type" as="xs:string">
	<param name="entity" as="node()" />
	<value-of select="per:entity-type-internal($entity,'.controller.impl',
		'ServerControllerImplBase')" />
</function>

<function name="per:entity-type-internal" as="xs:string">
	<param name="entity" as="node()" />
	<param name="package-insert" as="xs:string" />
	<param name="suffix" as="xs:string" />
	<value-of select="per:entity-type-internal(
			$entity,$entity/@name,$package-insert,$suffix)" />
</function>

<function name="per:entity-type-internal" as="xs:string">
	<param name="entity" as="node()" />
	<param name="entity-name" as="xs:string" />
	<param name="package-insert" as="xs:string" />
	<param name="suffix" as="xs:string" />
	<value-of select="concat($entity/ancestor::seaf:module/@package,
			'.server',$package-insert,'.',$entity-name,$suffix)" />
</function>

<!-- Full ref types -->

<function name="per:enum-ref-full-type" as="xs:string">
	<param name="enum-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="enum-def" 
		select="$merged-persistence-file//per:enum-def[@name=$enum-name]" />
	<if test="count($enum-def) != 1">
		<message 
			select="'Error locating persistence enumeration definition: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$enum-name)" />
		<message terminate="yes" 
			select="concat('  - referenced enumerations found = ',count($enum-def))" />
	</if>
	<value-of select="per:entity-full-type($enum-def)" />
</function>

<function name="per:entity-ref-full-type" as="xs:string">
	<param name="entity-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="verify-entity" 
		select="per:entity-ref($entity-name,$merged-persistence-file)" />
	<variable name="entity" 
		select="$merged-persistence-file//per:entity[@name=$entity-name]" />
	<value-of select="per:entity-full-type($entity)" />
</function>

<function name="per:entity-ref" as="node()">
	<param name="entity-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="entity" 
		select="$merged-persistence-file//per:entity[@name=$entity-name]" />
	<if test="count($entity) != 1">
		<message select="'Error locating entity definition: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$entity-name)" />
		<message terminate="yes" 
			select="concat('  - referenced entities found = ',count($entity))" />
	</if>
	<copy-of select="$entity" />
</function>

<function name="per:entity-criteria-ref-full-type" as="xs:string">
	<param name="entity-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="verify-entity" 
		select="per:entity-ref($entity-name,$merged-persistence-file)" />
	<variable name="entity" 
		select="$merged-persistence-file//per:entity[@name=$entity-name]" />
	<value-of select="per:entity-criteria-full-type($entity)" />
</function>

<!-- SQL names -->

<function name="per:table-name" as="xs:string">
	<param name="entity-name" as="xs:string" />
	<value-of select="concat('t_',seaf:convert-name-to-sql($entity-name))" />
</function>

<function name="per:sequence-name" as="xs:string">
	<param name="entity-name" as="xs:string" />
	<value-of select="concat('s_',seaf:convert-name-to-sql($entity-name))" />
</function>

<function name="per:column-name" as="xs:string">
	<param name="field-name" as="xs:string" />
	<value-of select="concat('c_',seaf:convert-name-to-sql($field-name))" />
</function>

<function name="per:fk-name" as="xs:string">
	<param name="from-entity" as="xs:string" />
	<param name="to-entity" as="xs:string" />
	<value-of select="concat('fk_',
		per:cut-sql-name(seaf:convert-name-to-sql($from-entity),13),
		'__',
		per:cut-sql-name(seaf:convert-name-to-sql($to-entity),11))" />
</function>

<function name="per:cut-sql-name" as="xs:string">
	<param name="name" as="xs:string" />
	<param name="length" as="xs:integer" />
	<variable name="tokens" select="tokenize($name,'_')" />
	<variable name="token-length" 
		select="($length - count($tokens) + 1) idiv count($tokens)" />
	<variable name="cut-tokens" 
		select="for $token in $tokens return substring($token,1,$token-length)" />
	<value-of select="string-join($cut-tokens,'_')" />
</function>

<!-- Java types -->

<function name="per:field-simple-property-type" as="xs:string">
	<param name="field" as="node()" />
	<choose>
	<when test="$field/per:string or $field/per:text">string</when>
	<when test="$field/per:int">int</when>
	<when test="$field/per:long">long</when>
	<when test="$field/per:float">float</when>
	<when test="$field/per:double">double</when>
	<when test="$field/per:boolean">boolean</when>
	<when test="$field/per:date">calendar</when>
	<when test="$field/per:time">calendar</when>
	<when test="$field/per:datetime">calendar</when>
	<when test="$field/per:timestamp">calendar</when>
	<otherwise>
		<value-of select="concat('ERROR! Unknown field type: ',
			(for $child in $field/* return local-name($child))[not(.='description')])" />
	</otherwise>
	</choose>
</function>

<function name="per:primary-key-java-type" as="xs:string">
	<param name="field" as="node()" />
	<choose>
	<when test="$field/per:string">String</when>
	<when test="$field/per:int">Integer</when>
	<when test="$field/per:long">Long</when>
	<otherwise>
		<message>
			<text />ERROR: Unsupported type on a primary key field <text />
				<value-of select="$field/@name" />
		</message>
		<message terminate="yes">
			<text />The following types are supported: string, int, long.<text />
		</message>
	</otherwise>
	</choose>
</function>

<function name="per:is-collection" as="xs:boolean">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<value-of select="(name($relation-side)='master') and 
		($relation/@type='one-to-many')" />
</function>

<function name="per:is-opposite-collection" as="xs:boolean">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<value-of select="(name($relation-side)='detail') and 
		($relation/@type='one-to-many')" />
</function>

<function name="per:opposite-relation-side" as="node()">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<choose>
	<when test="name($relation-side)='detail'">
		<copy-of select="$relation/per:master" />
	</when>
	<otherwise>
		<copy-of select="$relation/per:detail" />
	</otherwise>
	</choose>
</function>

<function name="per:related-entity-name" as="xs:string">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<variable name="opposite-side" 
		select="per:opposite-relation-side($relation-side,$relation)" />
	<value-of select="$opposite-side/@entity-name" />
</function>

<function name="per:opposite-relation-property-name" as="xs:string">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<variable name="field-name" select="$relation-side/@field-name" />
	<variable name="entity-field-name">
		<value-of select="seaf:first-lower($relation-side/@entity-name)" />
		<if test="per:is-opposite-collection($relation-side,$relation)">List</if>
	</variable>
	<value-of select="per:relation-property-name-internal(
		$field-name,$entity-field-name)" />
</function>

<function name="per:relation-property-name" as="xs:string">
	<param name="relation-side" as="node()" />
	<param name="relation" as="node()" />
	<variable name="field-name" 
		select="per:opposite-relation-side($relation-side,$relation)/@field-name" />
	<variable name="entity-field-name">
		<value-of select="seaf:first-lower(per:related-entity-name(
			$relation-side,$relation))" />
		<if test="per:is-collection($relation-side,$relation)">List</if>
	</variable>
	<value-of select="per:relation-property-name-internal(
		$field-name,$entity-field-name)" />
</function>

<function name="per:relation-property-name-internal" as="xs:string">
	<param name="field-name" as="xs:string*" />
	<param name="entity-field-name" as="xs:string" />
	<value-of>
		<choose>
		<when test="$field-name">
			<value-of select="$field-name" />
		</when>
		<otherwise>
			<value-of select="$entity-field-name" />
		</otherwise>
		</choose>
	</value-of>
</function>

<function name="per:persistent-object-type-label" as="xs:string">
	<param name="node" as="node()" />
	<choose>
	<when test="$node/@code-table">code table</when>
	<otherwise>entity</otherwise>
	</choose>
</function>

<function name="per:persitent-object-pk-field" as="node()">
	<param name="entity" as="node()" />
	<variable name="fields" select="per:entity-fields($entity,())" />
	<variable name="pk-fields" 
		select="$fields[@status='primary-key-assigned' or 
			@status='primary-key-generated']" />
	<if test="count($pk-fields) != 1">
		<message select="'Error locating primary key field: exactly 1 expected'"/>
		<message select="concat(' entity = ',$entity/@name)" />
		<message select="concat(' primary key fields found = ',count($pk-fields))"
			terminate="yes" />
	</if>
	<copy-of select="$pk-fields" />
</function>

<function name="per:persitent-object-pk-type" as="xs:string">
	<param name="entity" as="node()" />
	<variable name="pk-field" select="per:persitent-object-pk-field($entity)" />
	<value-of select="per:primary-key-java-type($pk-field)" />
</function>

<function name="per:bean-interface" as="xs:string">
	<param name="entity" as="node()" />
	<value-of>
		<choose>
		<when test="$entity/@code-table='code-based'">
			<text>net.sf.seaf.persistence.CodeBasedCodeTable</text>
		</when>
		<when test="$entity/@code-table='time-bound'">
			<text>net.sf.seaf.persistence.TimeBoundCodeTable</text>
		</when>
		<otherwise>
			<text>net.sf.seaf.persistence.Entity&lt;</text>
			<value-of select="per:persitent-object-pk-type($entity)" />
			<text>></text>
		</otherwise>
		</choose>
	</value-of>
</function>

<function name="per:entity-fields" as="node()*">
	<param name="entity" as="node()" />
	<param name="merged-persistence-file" as="node()*" />
	<if test="not($entity/per:field[@status='primary-key-assigned' 
		or @status='primary-key-generated']
		or $entity/@code-table='code-based')">
		<per:field name="id" status="primary-key-generated">
			<per:description>Surrogate generated primary key.</per:description>
			<per:long />
		</per:field>
	</if>
	<variable name="code-field" select="$entity/per:field[@status='code-table-code']" />
	<if test="$entity/@code-table='code-based' and not($code-field)">
		<per:field name="code" status="primary-key-assigned">
			<per:description>Code.</per:description>
			<per:string length="1024" />
		</per:field>
	</if>
	<if test="$entity/@code-table='code-based' and $code-field">
		<copy-of select="per:copy-code-table-field($code-field,'primary-key-assigned')" />
	</if>
	<if test="$entity/@code-table='time-bound' and not($code-field)">
		<per:field name="code" status="mandatory">
			<per:description>Code.</per:description>
			<per:string length="1024" />
		</per:field>
	</if>
	<if test="$entity/@code-table='time-bound' and $code-field">
		<copy-of select="per:copy-code-table-field($code-field,'mandatory')" />
	</if>
	<variable name="label-field" select="$entity/per:field[@status='code-table-label']" />
	<if test="$entity/@code-table and not($label-field)">
		<per:field name="label">
			<per:description>Default label.</per:description>
			<per:string length="1024" />
		</per:field>
	</if>
	<if test="$entity/@code-table and $label-field">
		<copy-of select="per:copy-code-table-field($label-field,'optional')" />
	</if>
	<variable name="valid-field" select="$entity/per:field[@status='code-table-valid']" />
	<if test="$entity/@code-table='code-based' and not($valid-field)">
		<per:field name="valid">
			<per:description>Validity flag of the record.</per:description>
			<per:description>Records with valid=true are valid, other records are invalid.</per:description>
			<per:boolean />
		</per:field>
	</if>
	<if test="$entity/@code-table and $valid-field">
		<copy-of select="per:copy-code-table-field($valid-field,'optional')" />
	</if>
	<variable name="valid-from-field" select="$entity/per:field[@status='code-table-valid-from']" />
	<if test="$entity/@code-table='time-bound' and not($valid-from-field)">
		<per:field name="validFrom" status="mandatory">
			<per:description>The date on which this record becomes valid. The first date of validity period.</per:description>
			<per:date />
		</per:field>
	</if>
	<if test="$entity/@code-table and $valid-from-field">
		<copy-of select="per:copy-code-table-field($valid-from-field,'mandatory')" />
	</if>
	<variable name="valid-until-field" select="$entity/per:field[@status='code-table-valid-until']" />
	<if test="$entity/@code-table='time-bound' and not($valid-until-field)">
		<per:field name="validUntil" status="optional">
			<per:description>The date after which this record is not valid anymore. The last date of validity period.</per:description>
			<per:date />
		</per:field>
	</if>
	<if test="$entity/@code-table and $valid-until-field">
		<copy-of select="per:copy-code-table-field($valid-until-field,'optional')" />
	</if>
	<copy-of select="$entity/per:field[not(@status='code-table-code' or @status='code-table-label' 
		or @status='code-table-valid' or @status='code-table-valid-from' 
		or @status='code-table-valid-until')]" />
	<if test="not($entity/per:field[@status='version'])">
		<per:field name="versn" status="version">
			<per:description>The version column for optimistic locking mechanism.</per:description>
			<per:long />
		</per:field>
	</if>
	<variable name="entity-name" select="$entity/@name" />
	<for-each select="$merged-persistence-file//per:detail[@entity-name=$entity-name]">
		<variable name="related-entity-name" select="per:related-entity-name(.,..)" />
		<variable name="related-entity" 
			select="$merged-persistence-file//per:entity[@name=$related-entity-name]" />
		<variable name="related-pk-field" select="per:persitent-object-pk-field($related-entity)" />
		<variable name="related-pk-type" select="per:field-simple-property-type($related-pk-field)" />
		<per:field name="{per:relation-property-name(.,..)}Id" status="criteria-only">
			<per:description>
				<value-of select="concat('Criterion on master entity primary key ',
					$related-entity-name,'.',$related-pk-field/@name,'.')" />
			</per:description>
			<element name="{$related-pk-type}" namespace="http://seaf.sf.net/seaf-persistence" />
		</per:field>
	</for-each>
</function>

<function name="per:copy-code-table-field" as="node()*">
	<param name="field" as="node()" />
	<param name="status" as="xs:string" />
	<element name="field" namespace="http://seaf.sf.net/seaf-persistence">
		<attribute name="name" select="$field/@name" />
		<attribute name="status" select="$status" />
		<copy-of select="$field/*" />
	</element>
</function>

<function name="per:code-table-code-field" as="node()">
	<param name="code-table-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="entity" select="per:entity-ref($code-table-name,$merged-persistence-file)" />
	<variable name="field" select="$entity/per:field[@status='code-table-code']" />
	<choose>
	<when test="$field">
		<copy-of select="$field" />
	</when>
	<otherwise>
		<variable name="fields" select="per:entity-fields($entity,())" />
		<copy-of select="$fields[@name='code']" />
	</otherwise>
	</choose>
</function>

</stylesheet>
