package net.sf.seaf.persistence.dozer;

import net.sf.seaf.factory.Factory;
import net.sf.seaf.factory.impl.DefaultInstantiatingFactory;
import net.sf.seaf.persistence.EnumEntity;
import net.sf.seaf.persistence.exception.PersistentEnumerationConversionException;

import org.dozer.CustomConverter;

/**
 * Dozer converter converting between {@link EnumEntity} and {@link Enum}.
 */
public class EnumEntityEnumConverter implements CustomConverter {

	public Object convert(Object existingDestinationFieldValue,
			Object sourceFieldValue,
			@SuppressWarnings("rawtypes") Class destinationClass,
			@SuppressWarnings("rawtypes") Class sourceClass) {
		if (null != sourceFieldValue) {
			if (EnumEntity.class.isAssignableFrom(sourceClass)
					&& Enum.class.isAssignableFrom(destinationClass)) {
				return convertEntityToEnum(sourceFieldValue, destinationClass);
			}
			if (EnumEntity.class.isAssignableFrom(destinationClass)
					&& Enum.class.isAssignableFrom(sourceClass)) {
				return convertEnumToEntity(sourceFieldValue, destinationClass);
			}
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	private Object convertEntityToEnum(Object sourceFieldValue,
			@SuppressWarnings("rawtypes") Class destinationClass) {
		EnumEntity<?> entity = (EnumEntity<?>) sourceFieldValue;
		return Enum.valueOf(destinationClass, entity.getId().toString());
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private Object convertEnumToEntity(Object sourceFieldValue,
			Class<?> destinationClass) {
		EnumEntity entity = (EnumEntity) factory
				.getInstanceOf(destinationClass);
		Class entityEnumClass = getEntityEnumClass(destinationClass);
		Enum entityEnumValue = Enum.valueOf(entityEnumClass,
				sourceFieldValue.toString());
		entity.setId(entityEnumValue);
		return entity;
	}

	private Class<?> getEntityEnumClass(Class<?> destinationClass) {
		String entityEnumClassName = destinationClass.getName() + "$Code";
		try {
			return Class.forName(entityEnumClassName);
		} catch (ClassNotFoundException e) {
			throw new PersistentEnumerationConversionException(
					"Cannot access enumeration of an enumerated persistent entity "
							+ entityEnumClassName, e);
		}
	}

	private Factory factory = new DefaultInstantiatingFactory();

	/**
	 * Set the factory used to instantiate EnumEntity instances.
	 * 
	 * @param factory
	 */
	public void setFactory(Factory factory) {
		this.factory = factory;
	}

}
