package net.sf.seaf.persistence.util;

import net.sf.seaf.exception.SeafClassLoadingException;
import net.sf.seaf.persistence.ServerController;
import net.sf.seaf.util.FactoryUtils;

/**
 * Utility methods for conversion between seaf persistence types.
 */
public class TypeConversionUtil {

	private static TypeConversionUtil instance = new TypeConversionUtil();

	/**
	 * Returns a default instance of TypeConversionUtil.
	 * 
	 * @return A default instance of TypeConversionUtil
	 */
	public static TypeConversionUtil getInstance() {
		return instance;
	}

	/**
	 * Convert an entity type to the type of the corresponding controller.
	 * 
	 * @param entityType
	 *            Type of an entity
	 * @return The type of the corresponding controller
	 */
	public Class<? extends ServerController> entityToController(Class<?> entityType) {
		String pkg = entityType.getPackage().getName();
		pkg = pkg.substring(0, pkg.length() - 6) + "controller.";
		String controllerTypeName = pkg + entityType.getSimpleName()
				+ "ServerController";
		try {
			Class<? extends ServerController> controllerType = FactoryUtils
					.getInstance().getClassForName(controllerTypeName,
							ServerController.class);
			return controllerType;
		} catch (SeafClassLoadingException e) {
			throw new SeafClassLoadingException(
					"Cannot load controller class of entity " + entityType, e
							.getCause());
		}
	}

}
